from pathlib import Path

import os
import click
import configparser

from .babel import (
    check_babel_configuration,
    extract_babel_messages,
    update_babel_translations,
    compile_babel_translations,
    prepare_babel_translation_dir,
    merge_catalogues_from_translation_dir,
)
from .i18next import (
    ensure_i18next_entrypoint,
    extract_i18next_messages,
    compile_i18next_translations, merge_catalogues_from_i18next_translation_dir,
)


@click.command(
    help="Generates and compiles localization messages. "
    "Reads configuration from setup.cfg and uses it to call babel and i18next. "
    "Expects setup.cfg in the current directory or you may pass the path to it as an argument."
)
@click.argument("setup_cfg", default="setup.cfg")
def main(setup_cfg):
    base_dir = Path(setup_cfg).resolve().parent
    os.chdir(base_dir)

    configuration = configparser.ConfigParser()
    configuration.read([setup_cfg])
    i18n_configuration = {
        k: [vv.strip() for vv in v.split("\n") if vv.strip()]
        for k, v in dict(configuration["oarepo.i18n"]).items()
    }
    print(i18n_configuration)

    check_babel_configuration(base_dir, i18n_configuration)

    translations_dir = prepare_babel_translation_dir(base_dir, i18n_configuration)

    extract_babel_messages(base_dir, i18n_configuration, translations_dir)

    update_babel_translations(translations_dir)

    for extra_translations in i18n_configuration.get("babel_input_translations", []):
        merge_catalogues_from_translation_dir(
            base_dir / extra_translations, translations_dir
        )

    for extra_i18next_translations in i18n_configuration.get("i18next_input_translations", []):
        merge_catalogues_from_i18next_translation_dir(
            base_dir / extra_i18next_translations, translations_dir
        )

    i18next_translations_dir = next(
        iter(i18n_configuration["i18next_output_translations"]), None
    )
    extract_i18next_messages(base_dir, i18n_configuration, i18next_translations_dir)
    merge_catalogues_from_translation_dir(
        base_dir / i18next_translations_dir / "messages", translations_dir
    )

    compile_babel_translations(translations_dir)
    compile_i18next_translations(
        translations_dir, i18n_configuration, base_dir / i18next_translations_dir
    )
    ensure_i18next_entrypoint(base_dir / i18next_translations_dir)


if __name__ == "__main__":
    main()



import json
import os
import shutil
import inspect
from pathlib import Path
from subprocess import check_call

import click
import polib

npm_proj_cwd = os.path.dirname(inspect.getfile(inspect.currentframe()))
npm_proj_env = dict(os.environ)


def extract_i18next_messages(base_dir: Path, i18n_configuration, translations_dir):
    npm_proj_env["LANGUAGES"] = ",".join(i18n_configuration["languages"] or ["en"])

    source_path_patterns = [
        os.path.join(base_dir / source_path, "**/*.{js,jsx,ts,tsx}")
        for source_path in i18n_configuration["i18next_source_paths"]
    ]

    output_path = base_dir / translations_dir

    # Make sure NPM project is installed & up-to-date
    click.secho("Installing / updating React-i18next dependencies", fg="green")
    check_call(
        ["npm", "install"],
        env=npm_proj_env,
        cwd=npm_proj_cwd,
    )

    # Extract JS translations strings
    click.secho(
        f"Extracting i18next  messages from sources matching {source_path_patterns} -> {output_path}",
        fg="green",
    )
    check_call(
        [
            "npm",
            "run",
            "extract_messages",
            "--",
            "--output",
            output_path,
            *source_path_patterns,
        ],
        env=npm_proj_env,
        cwd=npm_proj_cwd,
    )

    # Extract JS translations strings
    click.secho(
        f"Compiling i18next translations to PO files in: {output_path}",
        fg="green",
    )
    check_call(
        [
            "npm",
            "run",
            "compile_languages",
            "--",
            output_path,
        ],
        env=npm_proj_env,
        cwd=npm_proj_cwd,
    )


def ensure_i18next_entrypoint(i18next_translations_dir: Path):
    # check if i18next.js exists and if it does not, create it
    i18next_entrypoint = i18next_translations_dir / "i18next.js"

    if not i18next_entrypoint.exists():
        shutil.copy(Path(__file__).parent / "i18next.js", i18next_entrypoint)
        click.secho(f"Created i18next.js in {i18next_entrypoint}", fg="green")


def compile_i18next_translations(
    translations_dir, i18n_configuration, output_translations_dir
):
    npm_proj_env["LANGUAGES"] = ",".join(i18n_configuration["languages"] or ["en"])

    click.secho(f"Compiling i18next messages in {translations_dir}", fg="green")
    check_call(
        [
            "npm",
            "run",
            "compile_catalog",
            "--",
            translations_dir,
            output_translations_dir,
        ],
        env=npm_proj_env,
        cwd=npm_proj_cwd,
    )


def merge_i18next_catalogues(source_catalogue_file: Path, target_catalogue_file: Path):
    source_catalogue = json.loads(source_catalogue_file.read_text())
    target_catalogue = polib.pofile(str(target_catalogue_file))

    target_catalogue_by_msgid = {entry.msgid: entry for entry in target_catalogue}

    for key, value in source_catalogue.items():
        if key in target_catalogue_by_msgid:
            if value:
                target_catalogue_by_msgid[key].msgstr = value
        else:
            target_catalogue.append(polib.POEntry(msgid=key, msgstr=value))

    target_catalogue.save(str(target_catalogue_file))


def merge_catalogues_from_i18next_translation_dir(source_translation_dir, target_translation_dir):
    for source_catalogue_file in source_translation_dir.glob("*/translations.json"):
        click.secho(
            f"Merging i18next {source_catalogue_file} into {target_translation_dir}", fg="yellow"
        )
        language = source_catalogue_file.parent.name

        target_catalogue_file = target_translation_dir / language / "LC_MESSAGES" / "messages.po"
        if target_catalogue_file.exists():
            merge_i18next_catalogues(source_catalogue_file, target_catalogue_file)
        else:
            click.secho(f"Target catalogue file {target_catalogue_file} does not exist, "
                        f"can not merge {source_catalogue_file}", fg="red")



import shutil
import sys
from pathlib import Path
from subprocess import check_output
import click

try:
    from babel.messages.frontend import CommandLineInterface
except ImportError:
    click.secho(
        "Babel is not installed in the current virtualenv. "
        'Please install it using "pip install babel" or '
        "add babel, polib and jinja2 to your dev depedencies.",
        fg="red",
    )
    sys.exit(1)

try:
    import polib
except ImportError:
    click.secho(
        "polib is not installed in the current virtualenv. "
        'Please install it using "pip install polib" or '
        "add babel, polib and jinja2 to your dev depedencies.",
        fg="red",
    )
    sys.exit(1)


try:
    import jinja2
except ImportError:
    click.secho(
        "Jinja2 is not installed in the current virtualenv. "
        'Please install it using "pip install jinja2" or '
        "add babel, polib and jinja2 to your dev depedencies.",
        fg="red",
    )
    sys.exit(1)


def check_babel_configuration(base_dir, i18n_configuration):
    babel_ini_file = base_dir / "babel.ini"
    # check if babel.ini exists and if it does not, create it
    if not babel_ini_file.exists():
        shutil.copy(Path(__file__).parent / "babel.ini", babel_ini_file)
        click.secho(f"Created babel.ini in {base_dir}", fg="green")


def prepare_babel_translation_dir(base_dir, i18n_configuration) -> Path:
    translations_dir = base_dir / i18n_configuration["babel_output_translations"][0]

    if not translations_dir.exists():
        translations_dir.mkdir(parents=True)
        click.secho(f"Created {translations_dir}", fg="green")

    for language in i18n_configuration.get("languages", ("cs", "en")):
        catalogue_dir = translations_dir / language / "LC_MESSAGES"
        if not catalogue_dir.exists():
            catalogue_dir.mkdir(parents=True)
            click.secho(f"Created {catalogue_dir}", fg="green")
        messages = catalogue_dir / "messages.po"
        if not messages.exists():
            messages.touch()
            click.secho(f"Created {messages}", fg="green")

    return translations_dir


def extract_babel_messages(base_dir, i18n_configuration, translations_dir):
    babel_ini_file = base_dir / "babel.ini"
    # extract messages
    jinjax_extra_source = str(
        translations_dir.relative_to(base_dir) / "jinjax_messages.jinja"
    )

    sources = [
        x
        for x in i18n_configuration["babel_source_paths"] + [jinjax_extra_source]
        if x.strip()
    ]

    translations_file = str(translations_dir / "messages.pot")

    click.secho(
        f"Extracting babel messages from {', '.join(sources)} -> {translations_file}"
    )

    find_jinjax_strings = """grep -E -hori '[^\{]\{\s_\(.*\)\s\}[^\}]'"""
    search_sources = f"{find_jinjax_strings} {' '.join(sources)}"
    reformat_for_babel = """awk '{print "{" substr($0, 2, length($0) - 2) "}"}'"""

    extract_jinjax_strings = (
        f"{search_sources} | {reformat_for_babel} > {jinjax_extra_source}"
    )
    check_output(extract_jinjax_strings, shell=True)

    CommandLineInterface().run(
        [
            "pybabel",
            "extract",
            "-F",
            babel_ini_file,
            "-k",
            "lazy_gettext",
            "-o",
            translations_file,
            *sources,
        ]
    )

    return translations_dir


def update_babel_translations(translations_dir):
    click.secho(f"Updating messages in {translations_dir}", fg="green")
    for catalogue_file in translations_dir.glob("*/LC_MESSAGES/*.po"):
        merge_catalogues(
            translations_dir / "messages.pot",
            translations_dir / catalogue_file.relative_to(translations_dir),
        )


def compile_babel_translations(translations_dir):
    click.secho(f"Compiling messages in {translations_dir}", fg="green")

    CommandLineInterface().run(["pybabel", "compile", "-f", "-d", translations_dir])
    click.secho(f"Done", fg="green")


def merge_catalogues(source_catalogue_file: Path, target_catalogue_file: Path):
    source_catalogue = polib.pofile(str(source_catalogue_file))
    target_catalogue = polib.pofile(str(target_catalogue_file))
    target_catalogue_by_msgid = {entry.msgid: entry for entry in target_catalogue}

    for entry in source_catalogue:
        if entry.msgid not in target_catalogue_by_msgid:
            target_catalogue.append(entry)
            target_catalogue_by_msgid[entry.msgid] = entry
        elif (
            entry.msgstr
            and entry.msgstr != target_catalogue_by_msgid[entry.msgid].msgstr
        ):
            target_catalogue_by_msgid[entry.msgid].msgstr = entry.msgstr

    target_catalogue.save(str(target_catalogue_file))
    target_catalogue.save_as_mofile(str(target_catalogue_file.with_suffix(".mo")))


def merge_catalogues_from_translation_dir(
    source_translation_dir: Path, target_translation_dir: Path
):
    for catalogue_file in source_translation_dir.glob("*/LC_MESSAGES/*.po"):
        click.secho(
            f"Merging {catalogue_file} into {target_translation_dir}", fg="yellow"
        )
        merge_catalogues(
            catalogue_file,
            target_translation_dir / catalogue_file.relative_to(source_translation_dir),
        )
